@@ -17,3 +17,5 @@
   (hinted [^int i])
   (hinted [^String s]))
 
+(defprotocol LongsHintedProto
+  (^longs longs-hinted [_]))
